@@ -29,11 +29,12 @@ if TYPE_CHECKING:
     from zarr.abc.store import Store
     from zarr.core.buffer.core import NDArrayLikeOrScalar
     from zarr.core.common import MemoryOrder
+    from zarr.types import AnyAsyncArray
 
 
 @dataclass(frozen=True)
 class _AsyncArrayProxy:
-    array: AsyncArray[Any]
+    array: AnyAsyncArray
 
     def __getitem__(self, selection: BasicSelection) -> _AsyncArraySelectionProxy:
         return _AsyncArraySelectionProxy(self.array, selection)
@@ -41,7 +42,7 @@ class _AsyncArrayProxy:
 
 @dataclass(frozen=True)
 class _AsyncArraySelectionProxy:
-    array: AsyncArray[Any]
+    array: AnyAsyncArray
     selection: BasicSelection
 
     async def get(self) -> NDArrayLikeOrScalar:
